library(stats)

baseini <- function(x, y, data=sys.parent(), puloc, method=c("shift", "select"), smooth=c("spline", "loess"), control=list(shift=-1, spline=list(nb=~x, rk=cubic(x), spar="v", limnla=c(-1.5,0)), loess=list(span=0.75), select=list(npl=2, npr=3)))
	{
        
        ctr <- list(shift=quote(-1), spline=list(nb=~x, rk=quote(cubic(x)), spar="v", limnla=c(-1.5,0)), loess=list(span=0.75), select=list(npl=2, npr=3))
        
        xcall <- match.call()
	if(!is.null(xcall$control)) 
	{
        if(is.null(xcall$control$spline) & is.null(xcall$control$loess) & is.null(xcall$control$select)) stop("control argument must be a list with at least one of the names: spline, loess and select")
        if( is.null(xcall$control$spline$nb) != is.null(xcall$control$spline$rk) )
               stop("for spline smoothing, nb and rk have to specified at the same time with the same covariates")
            
	if(!is.null(xcall$control$spline))
          ctr$spline[names(xcall$control$spline)] <- xcall$control$spline
  
        if(!is.null(xcall$control$loess))
     	   ctr$loess[names(xcall$control$loess)] <- xcall$control$loess
        if(!is.null(xcall$control$shift))
     	   ctr$shift[names(xcall$control$shift)] <- xcall$control$shift
	if(!is.null(xcall$control$select))
   	   ctr$select[names(xcall$control$select)] <- xcall$control$select
        }
        
        
        res <- deparse(xcall$y)
        ind <- deparse(xcall$x)
        data.orig <- data
        x <- eval(xcall$x, data.orig)
        y <- eval(xcall$y, data.orig) 
      
        puloc <- xscale(puloc, x)

        data <- data.frame(x, y, xscale(x))
        varName <- "x"
        dimnames(data)[[2]] <- c(ind, res, varName)
   
        rkCall <- ctr$spline$rk
        tt <- x

        if(!is.null(xcall$control$spline$nb))
        {
            rkCall <- xcall$control$spline$rk
            varName <- unique(all.vars(rkCall))
            tt <- xscale(eval(as.name(varName), env=data.orig))
            data <- data.frame(x, y, tt)
            dimnames(data)[[2]] <- c(ind, res, varName)
            }

        data[,ind] <-  xscale(x)
       
        if(method=="shift" & smooth=="spline"){
                rkmat <- eval(rkCall, env=data)
              	Bline <- dsidr(y, rkmat, model.matrix(as.formula(ctr$spline$nb), data=data), limnla=eval(ctr$spline$limnla), vmu=eval(ctr$spline$spar))$fit
                if(eval(ctr$shift) < 0)
        		Bline <- Bline-Bline[shiftit(puloc,data)$idx]+shiftit(puloc,data)$c
                else
                         Bline <- Bline - eval(ctr$shift)
        }
        if(method=="shift" & smooth=="loess"){
                Bline <- loess(y~x, span=eval(ctr$loess$span))$fit
                if(eval(ctr$shift) < 0)
                   Bline <- Bline-Bline[shiftit(puloc,data)$idx]+shiftit(puloc,data)$c
                else
                   Bline <- Bline - eval(ctr$shift)
        }
        if(method=="select"){
	        d <- x[2]-x[1]
		xs <- x
		ys <- y
                ts <- tt
		for (i in 1:length(puloc)) {
		  index <- xs<puloc[i]-eval(ctr$select$npl)*d | xs>puloc[i]+eval(ctr$select$npr)*d
  		  xs <- xs[index]
  		  ys <- ys[index]
                  ts <- ts[index]
                }

                if(smooth=="spline"){
                   xsscale <- xscale(ts)
                   xscale <- xscale(tt)
                   data.s <- data.frame(xscale(xs), ys, xscale(ts))
                   dimnames(data.s)[[2]] <- c(ind, res, varName)
                   rkmat <- eval(rkCall, env=data.s)
 		   fit <-  dsidr(ys, rkmat, model.matrix(as.formula(ctr$spline$nb), data=data.s), limnla=eval(ctr$spline$limnla), vmu=eval(ctr$spline$spar))
                   T <- cbind(rep(1, length(xscale)), xscale)
                   data.used <- data.frame(xscale)
                   dimnames(data.used)[[2]] <- varName
                   newdata <- data.frame(xsscale)
                   dimnames(newdata)[[2]] <- varName
                   Bline <- as.vector(T%*%fit$d + rkEval(rkCall, data.used, newdata)[[1]]%*%fit$c)
                }
                if(smooth=="loess")   
                {Bline <-  predict(loess(ys~xs, span=eval(ctr$loess$span), control=loess.control(surface="direct")), data.frame(xs=x))
                }
       }
return(Bline)
}
         
#baseini.control <- function(nb=~x, rk=cubic(x), spar="m", limnla=c(-1.5, 0), span=0.75, npl=2, npr=3)
#        list(spline=list(nb=nb, rk=rk, spar=spar, limnla=limnla), loess=list(span=span), select=list(npl=npl, npr=npr))

"shiftit" <- function(x, data)
{
# x: initial locations; data: dataframe with 2-col (ind, res)
	
        if(length(x) == 1)
        {  filter.dat <- data[ data[,1] <= x, ]
           if( (max(data[,1])-x) > (x - min(data[,1])) ) filter.dat <- data[ data[,1] >=x, ]
        }
        else
        {  xx <- diff(x)
	   flg <- order(xx)[length(x) - 1]
	   filter.dat <- data[data[, 1] >= x[flg] & data[, 1] <= x[flg + 1],  ]
        }
	min.dat <- filter.dat[filter.dat[, 2] == min(filter.dat[, 2]),  ]
	c.min <- min.dat[, 2]
	min.idx <- as.numeric(dimnames(min.dat)[[1]])
	return(list(c = c.min, idx = min.idx))
	}
