\name{CLUSTER}
\alias{CLUSTER}
\title{CLUSTER Method for Detecting Pulse Locations.}
\description{Detect pulse locations using the CLUSTER method}
\usage{CLUSTER(x, y, data, sd=.07*mean(y), nnadir=2, npeak=3, alpha)}
\arguments{
\item{x}{a vector of observation time points.}  
\item{y}{a vector of hormone concentrations.} 
\item{data}{a data frame containing the variables occurring in the \var{x} 
and \var{y} arguments. If this option is not specified,  the variables 
should be on the search list. Missing values are not allowed.} 
\item{sd}{standard deviation for the pooled $t$ test. Default is 7\% of the 
mean response.} 
\item{nnadir}{number of nadir points used in the $t$ statistics. Default is 2.} 
\item{nnpeak}{number of peak points used in the $t$ statistics. Default is 3.}
\item{alpha}{Significance level of the $t$ statistics.}
}
\details{CLUSTER uses $t$ test between two overlapping consecutive 
sets of points to identify significant increases and decreases in 
observations. The default for the standard deviation \var{sd} is by 
no means standard. Users are recommended to provide an accurate estimate 
instead of using the default.}
\value{A vector of pulse locations.}
\references{
    Veldhuis, J. D. and Johnson, M. L. (1986), Cluster analysis: 
    a simple versatile and robust algorithm for endocrine pulse detection,
    \emph{American Journal of Physiology}, \bold{250}, E486-E493.
}
\author{Yu-Chieh Yang, Anna Liu, Yuedong Wang}
\seealso{\code{\link{pulini}}}
\examples{
pl4 <- CLUSTER(time, conc, data=acth, sd=.05*mean(acth$conc), alpha=.2)
}
